package jp.hishidama.eval.exp;

import java.util.*;

import jp.hishidama.eval.EvalException;
import jp.hishidama.eval.Expression;
import jp.hishidama.eval.ExpRuleFactory;
import jp.hishidama.eval.func.*;
import jp.hishidama.eval.var.*;
import junit.framework.TestCase;

/**
 * doubleZ̃eXg.
 * 
 * @author <a target="hishidama"
 *         href="http://www.ne.jp/asahi/hishidama/home/soft/java/eval.html">Ђ</a>
 */
public class ExpressionDoubleTest extends TestCase {

	/*
	 * 'jp.hishidama.eval.Expression.eval()' ̂߂̃eXgE\bh
	 */
	public void testEval() {
		testEval("", null, null, 0);
		testEval(" 12.34 ", null, null, 12.34);
		testEval("1.2+3.4", null, null, 4.6);
		testEval("12.3+-+4", null, null, 8.3);
		testEval("12.3*4", null, null, 49.2);
		testEval("1+2*3-4.5", null, null, 2.5);
		testEval("(1+2)*3.5", null, null, 10.5);
		testEval("1+10.5%4", null, null, 1 + 10.5 % 4);
		testEval("10 / 3 * 3", null, null, 10);
		testEval("3 * 10 / 3", null, null, 10);
		testEval("-2 ** 3**2", null, null, -64);
	}

	public void testHex() {
		testEval("0x1234567890", null, null, 0x1234567890l);
		testEval("0xabcdef", null, null, 0xabcdef);
		testEval("0XABCDEF", null, null, 0xabcdef);
		testEval(" 0xfl ", null, null, 0xf);
		testEval(" 0xfL ", null, null, 0xf);
	}

	public void testBin() {
		testEval("0b1110", null, null, 0xe);
		testEval("0B0111", null, null, 7);
		testEval("0B0111l", null, null, 7);
		testEval("0B0111L", null, null, 7);
	}

	public void testOct() {
		testEval("0o12345670", null, null, 012345670);
		testEval("0O12345670", null, null, 012345670);
		testEval("0O12345670l", null, null, 012345670);
		testEval("0O12345670L", null, null, 012345670);
	}

	public void testShift() {
		testEval("1<<8", null, null, 1 << 8);
		testEval("-8>>2", null, null, -8L >> 2);
		testEval("-8>>>2", null, null, 8L >>> 2);
		testEval("1+2<<3-1", null, null, 1 + 2 << 3 - 1);
	}

	public void testBit() {
		testEval("~123", null, null, ~123L);
		testEval("1 | 3 ^ 5 & 7", null, null, 1 | 3 ^ 5 & 7);
	}

	public void test() {
		testEval("(1)", null, null, 1);
		testEval("((2))", null, null, 2);
		testParse("(3))");
		testParse("((99)");
	}

	public void testError() {
		testParse("12 89");
		testParse("12+");
		testParse("12//4");
	}

	public void testIf() {
		testEval("1?2:3", null, null, 2);
		testEval("0?2:3", null, null, 3);
		testParse("1:2?3");
		testEval("0?2:1?4:5", null, null, 4); // false? 2 : (true ?4:5)
		testEval("0?2:0?4:5", null, null, 5); // false? 2 : (false?4:5)
		testEval("1?2:1?4:5", null, null, 2); // true ? 2 : (true ?4:5)
		testEval("1?2?3:4:5", null, null, 3);
		testEval("1?0?3:4:5", null, null, 4);
		testEval("0?2?3:4:5", null, null, 5);
		testEval("(1?2:0)?3:4", null, null, 3);
		testParse("2?:3");
		testParse("2?3:");
	}

	public void testCond() {
		testEval("2==2", null, null, 1);
		testEval("2==3", null, null, 0);
		testEval("2!=3", null, null, 1);
		testEval("3!=3", null, null, 0);
		testEval("2<3", null, null, 1);
		testEval("3<3", null, null, 0);
		testEval("2<=3", null, null, 1);
		testEval("3<=3", null, null, 1);
		testEval("3<=2", null, null, 0);
		testEval("4>3", null, null, 1);
		testEval("4>4", null, null, 0);
		testEval("4>=3", null, null, 1);
		testEval("4>=4", null, null, 1);
		testEval("3>=4", null, null, 0);
		testEval("!0", null, null, 1);
		testEval("!1", null, null, 0);
		testEval("!2", null, null, 0);
	}

	public void testAndOr() {
		testEval("2&&3", null, null, 3);
		testEval("0&&3", null, null, 0);
		testEval("2&&0", null, null, 0);
		testEval("2||3", null, null, 2);
		testEval("0||3", null, null, 3);
		testEval("0||0", null, null, 0);
		testEval("0||2&&3", null, null, 3);
		testEval("1||2&&3", null, null, 1);
	}

	public void testSign() {
		testEval("-1", null, null, -1);
		testEval("+23", null, null, 23);
		testEval("!0", null, null, 1);
		testEval("~0", null, null, ~0);
		testEval("-+-123", null, null, 123);
		testEval("-+!0", null, null, -1);
	}

	public void testVar() {
		testEval("1+0abc+2", null, null);
		testEval("abc", null, null);
		Map map = new HashMap();
		map.put("abc", new Double(12));
		testEval("abc", map, null, 12);
		testEval("1+ abc+2", map, null, 15);
		testEval("1+ def+2", map, null);
		map.put("def", new Float(34));
		testEval("1+ def+2", map, null, 37);
	}

	public void testArray() {
		Double[] a = new Double[4];
		a[1] = new Double(11);
		a[2] = new Double(22);
		Map map = new HashMap();
		map.put("a", a);
		testEval("a[1]", map, null, 11);
		testEval("a[3]=a[1]+a[2]", map, null, 33);
		testEval("a[4]", map, null);

		testEval("a[1]+=33", map, null, 44);
		System.out.println(a[1]);

		Double[][] b = new Double[2][];
		b[0] = new Double[3];
		b[1] = new Double[3];
		b[1][0] = new Double(100);
		b[1][1] = new Double(101);
		b[1][2] = new Double(102);
		map.put("b", b);
		map.put("j", new Long(2));
		testEval("b[1][j-1]", map, null, 101);
		testEval("a[1][0]", map, null);
		testEval("b[0][0]=b[1][0]+b[1][2]", map, null, 202);
		testEval("b[0][0]", map, null, 202);
	}

	public void testLet() {
		Map map = new HashMap();
		testEval("a=100+11", map, null, 111);
		assertVar(111, map, "a");
		testEval("a=b=12", map, null, 12);
		assertVar(12, map, "a");
		assertVar(12, map, "b");
		testEval("a=1+(b=12)", map, null, 13);
		assertVar(13, map, "a");
		assertVar(12, map, "b");
		testEval("a=1+b=2", map, null);

		map.put("a", new Double(100));
		map.put("b", new Double(10));
		testEval("a+=b+=1", map, null, 111);
		assertVar(111, map, "a");
		assertVar(11, map, "b");

		map.put("a", new Double(100));
		map.put("b", new Double(10));
		testEval("a-=b-=1", map, null, 91);
		assertVar(91, map, "a");
		assertVar(9, map, "b");

		map.put("a", new Double(2));
		map.put("b", new Double(3));
		testEval("a*=b*=4", map, null, 24);
		assertVar(24, map, "a");
		assertVar(12, map, "b");

		map.put("a", new Double(24));
		map.put("b", new Double(6));
		testEval("a/=b/=2", map, null, 8);
		assertVar(8, map, "a");
		assertVar(3, map, "b");

		map.put("a", new Double(10));
		map.put("b", new Double(10));
		testEval("a%=b%=6", map, null, 2);
		assertVar(2, map, "a");
		assertVar(4, map, "b");

		map.put("a", new Double(1));
		map.put("b", new Double(2));
		testEval("a<<=b<<=1", map, null, 1 << 4);
		assertVar(1 << 4, map, "a");
		assertVar(4, map, "b");

		map.put("a", new Double(1 << 8));
		map.put("b", new Double(1 << 4));
		testEval("a>>=b>>=3", map, null, 1 << 6);
		assertVar(1 << 6, map, "a");
		assertVar(1 << 1, map, "b");

		map.put("a", new Double(-1));
		map.put("b", new Double(-1));
		double b = 1d / Math.pow(2, 63);
		double a = 1d / Math.pow(2, b);
		testEval("a>>>=b>>>=63", map, null, a);
		assertVar(a, map, "a");
		assertVar(b, map, "b");

		map.put("a", new Double(0x0f7f));
		map.put("b", new Double(0x0ff0));
		testEval("a&=b&=255", map, null, 0x70);
		assertVar(0x70, map, "a");
		assertVar(0xf0, map, "b");

		map.put("a", new Double(0xf000));
		map.put("b", new Double(0x0f00));
		testEval("a|=b|=255-15", map, null, 0xfff0);
		assertVar(0xfff0, map, "a");
		assertVar(0x0ff0, map, "b");

		map.put("a", new Double(0xffff));
		map.put("b", new Double(0x0f0f));
		testEval("a^=b^=255", map, null, 0xf00f);
		assertVar(0xf00f, map, "a");
		assertVar(0xff0, map, "b");

		map.put("a", new Double(2));
		map.put("b", new Double(2));
		testEval("a**=b**=3", map, null, 256);
		assertVar(256, map, "a");
		assertVar(8, map, "b");
	}

	public void testComma() {
		Map map = new HashMap();
		testEval("aa=1,bb=0,cc=2,aa+bb+cc", map, null, 3);
		assertVar(1, map, "aa");
		assertVar(0, map, "bb");
		assertVar(2, map, "cc");
	}

	public void testIncBefore() {
		Map map = new HashMap();
		map.put("i", new Double(1));
		testEval("++i", map, null, 2);
		assertVar(2, map, "i");
		testEval("--i", map, null, 1);
		assertVar(1, map, "i");
		testEval("2+ ++i", map, null, 4);

		testEval("++--i", map, null);
	}

	public void testIncAfter() {
		Map map = new HashMap();
		map.put("i", new Double(1));
		testEval("i++", map, null, 1);
		assertVar(2, map, "i");
		testEval("i--", map, null, 2);
		assertVar(1, map, "i");
		testEval("2+ i++", map, null, 3);
		assertVar(2, map, "i");

		testEval("i++--", map, null);
		testEval("++i++", map, null);
		testEval("++i--", map, null);
	}

	public void testFunction() {
		testEval("test()", null, null, 0);
		testParse("test((1,1+1)");
		testParse("test(1,1+1))");
		Function func = new InvokeFunction();
		testEval("zzz()+foo(1)+test(2,3+4,5*6-7)", null, func, 0);

		Map map = new HashMap();
		map.put("a", new FieldTestA(10.1));
		testEval("a.getDbl()", map, func, 10.1);

		Function math = new MathFunction();
		testEval("sin(1)", null, math, Math.sin(1));
		testEval("cos(2)", null, math, Math.cos(2));
		testEval("pow(cos(3),2)+pow(sin(3),2)", null, math, 1);
	}

	public void testField() {
		FieldTestA a = new FieldTestA(111);
		Map map = new HashMap();
		map.put("a", a);
		testEval("a.n", map, null, 111);

		FieldTestC c = new FieldTestC();
		map.put("c", c);
		testEval("c.a.n", map, null, 222);
		testEval("c.a.n-c.z*2+a.n", map, null, -333);
	}

	public void testo[() {
		FieldTestA[] arr = new FieldTestA[3];
		arr[1] = new FieldTestA(10.1);
		Map map = new HashMap();
		map.put("a", arr);
		Function func = new InvokeFunction();
		testEval("a[1].dblValue", map, null, 12.3);
		testEval("a[1].getDbl()", map, func, 10.1);
		testEval("a[1].getB()[1].get()", map, func, 22);
	}

	/**
	 * ñeXg.
	 * 
	 * @param str
	 *            
	 * @param map
	 *            ϐ}bv
	 * @param func
	 *            ֐`
	 * @param expected
	 *            
	 */
	private void testEval(String str, Map map, Function func, double expected) {
		System.out.println("" + str + "");
		Expression exp = ExpRuleFactory.getDefaultRule().parse(str);
		if (map != null) {
			MapVariable var = new MapVariable();
			var.setMap(map);
			exp.setVariable(var);
		}
		// exp.dump(0);
		System.out.println(exp.toString());
		if (func != null) {
			exp.setFunction(func);
		}
		double ret = exp.evalDouble();
		System.out.println("= " + ret);
		assertEquals(expected, ret, DELTA);
	}

	/**
	 * ُñeXg.
	 * <p>
	 * ́E\͂EvalExceptionN͂B<br>
	 * ibZ[WǂɂẴ`FbN͏ȗj
	 * </p>
	 * 
	 * @param str
	 *            
	 */
	private void testParse(String str) {
		System.out.println("" + str + "");
		Expression exp;
		try {
			exp = ExpRuleFactory.getDefaultRule().parse(str);
		} catch (EvalException e) {
			System.out.println(e.toString());
			return;
		}
		// exp.dump(0);
		System.out.println(exp.toString());
		fail();
	}

	/**
	 * ُñeXg.
	 * <p>
	 * ]EvalExceptionN͂B<br>
	 * ibZ[WǂɂẴ`FbN͏ȗj
	 * </p>
	 * 
	 * @param str
	 *            
	 * @param map
	 *            ϐ}bv
	 * @param func
	 *            ֐`
	 */
	private void testEval(String str, Map map, Function func) {
		System.out.println("" + str + "");
		Expression exp = ExpRuleFactory.getDefaultRule().parse(str);
		if (map != null) {
			MapVariable var = new MapVariable();
			var.setMap(map);
			exp.setVariable(var);
		}
		// exp.dump(0);
		System.out.println(exp.toString());
		if (func != null) {
			exp.setFunction(func);
		}
		try {
			System.out.println("= " + exp.evalDouble());
		} catch (EvalException e) {
			System.out.println(e.toString());
			return;
		}
		fail();
	}

	/**
	 * ϐl̊mF.
	 * 
	 * @param expected
	 *            z肳l
	 * @param map
	 *            ϐ}bv
	 * @param name
	 *            ϐ
	 */
	private void assertVar(double expected, Map map, String name) {
		try {
			Double var = (Double) map.get(name);
			assertEquals(expected, var.doubleValue(), DELTA);
		} catch (Exception e) {
			throw new RuntimeException("assertVar(" + name + ")", e);
		}
	}

	/** double̔rɎgp臒l. */
	private static final double DELTA = Math.pow(10, -15);

}
