package jp.hishidama.eval.exp;

import java.util.*;

import jp.hishidama.eval.EvalException;
import jp.hishidama.eval.Expression;
import jp.hishidama.eval.ExpRuleFactory;
import jp.hishidama.eval.var.*;
import junit.framework.TestCase;

/**
 * DoubleœK̃eXg.
 * 
 * @author <a target="hishidama"
 *         href="http://www.ne.jp/asahi/hishidama/home/soft/java/eval.html">Ђ</a>
 */
public class OptimizeDoubleTest extends TestCase {

	/*
	 * 'jp.hishidama.eval.Expression.optimizeDouble()' ̂߂̃eXgE\bh
	 */
	public void testOptimize() {
		testOpt(" 1234 ", null, 1234);
		testOpt("12+34", null, 46);
		testOpt("123+-+4", null, 123 - 4);
		testOpt("123*4", null, 123 * 4);
		testOpt("1+2*3-4", null, 3);
		testOpt("(1+2)*3", null, 9);
		testOpt("1+10%4", null, 3);
		testOpt("10 / 3 * 3", null, 10);
		testOpt("3 * 10 / 3", null, 10);
		testOpt("1+2+a", null, "3.0 + a");
		testOpt("a+2+3", null, "a + 2 + 3");
	}

	public void testShift() {
		testOpt("1<<8", null, 1 << 8);
		testOpt("-8>>2", null, -8L >> 2);
		// +++testOpt("-8>>>2", null, -8L >>> 2);
		testOpt("1+2<<3-1", null, 1 + 2 << 3 - 1);
	}

	public void testBit() {
		testOpt("~123", null, ~123L);
		testOpt("1 | 3 ^ 5 & 7", null, 1 | 3 ^ 5 & 7);
	}

	public void test() {
		testOpt("(1)", null, 1);
		testOpt("((2))", null, 2);
		testOpt("(a)", null, "a");
		testOpt("((a))", null, "a");
	}

	public void testIf() {
		testOpt("1?2:3", null, 2);
		testOpt("0?2:3", null, 3);
		testOpt("0?2:1?4:5", null, 4); // false? 2 : (true ?4:5)
		testOpt("0?2:0?4:5", null, 5); // false? 2 : (false?4:5)
		testOpt("1?2:1?4:5", null, 2); // true ? 2 : (true ?4:5)
		testOpt("1?2?3:4:5", null, 3);
		testOpt("1?0?3:4:5", null, 4);
		testOpt("0?2?3:4:5", null, 5);
		testOpt("(1?2:0)?3:4", null, 3);
		testOpt("1?a:b", null, "a");
		testOpt("0?a:b", null, "b");
		testOpt("(1?2:0)?a:b", null, "a");
	}

	public void testCond() {
		testOpt("2==2", null, 1);
		testOpt("2==3", null, 0);
		testOpt("2!=3", null, 1);
		testOpt("3!=3", null, 0);
		testOpt("2<3", null, 1);
		testOpt("3<3", null, 0);
		testOpt("2<=3", null, 1);
		testOpt("3<=3", null, 1);
		testOpt("3<=2", null, 0);
		testOpt("4>3", null, 1);
		testOpt("4>4", null, 0);
		testOpt("4>=3", null, 1);
		testOpt("4>=4", null, 1);
		testOpt("3>=4", null, 0);
		testOpt("!0", null, 1);
		testOpt("!1", null, 0);
		testOpt("!2", null, 0);
	}

	public void testAndOr() {
		testOpt("2&&3", null, 3);
		testOpt("0&&3", null, 0);
		testOpt("2&&0", null, 0);
		testOpt("2||3", null, 2);
		testOpt("0||3", null, 3);
		testOpt("0||0", null, 0);
		testOpt("0||2&&3", null, 3);
		testOpt("1||2&&3", null, 1);
	}

	public void testSign() {
		testOpt("-1", null, -1);
		testOpt("+23", null, 23);
		testOpt("!0", null, 1);
		testOpt("~0", null, ~0);
		testOpt("-+-123", null, 123);
		testOpt("-+!0", null, -1);
	}

	public void testVar() {
		testOpt("abc", null, "abc");
		testOpt("1+0abc+2", null, "1 + 0abc + 2");
		Map map = new HashMap();
		map.put("abc", new Double(12));
		testOpt("abc", map, 12);
		testOpt("1+ abc+2", map, 15);
		testOpt("1+ def+2", map, "1 + def + 2");
		map.put("def", new Integer(34));
		testOpt("1+ def+2", map, 37);
	}

	public void testArray() {
		Map map = new HashMap();
		Double[] a = new Double[4];
		a[1] = new Double(11);
		a[2] = new Double(22);
		map.put("a", a);
		testOpt("a[1]", map, 11);
		testOpt("a[3]=a[1]+a[2]", map, "a[3] = 33.0");
		testOpt("a[4]", map, "a[4]");

		Double[][] b = new Double[2][];
		b[1] = new Double[3];
		b[1][0] = new Double(100);
		b[1][1] = new Double(101);
		b[1][2] = new Double(102);
		map.put("b", b);
		map.put("j", new Double(2));
		testOpt("b[1][j-1]", map, 101);
		testOpt("b[1][j-1]", null, "b[1][j - 1]");
	}

	public void testLet() {
		Map map = new HashMap();
		testOpt("a=100+11", map, "a = 111.0");
		// assertVar(111, map, "a");
		testOpt("a=b=12", map, "a = b = 12");
		// +++assertVar(12, map, "a");
		// assertVar(12, map, "b");
		testOpt("a=1+(b=12)", map, "a = 1 + (b = 12)");
		// +++assertVar(13, map, "a");
		// assertVar(12, map, "b");
		testOpt("a=1+b=2", map, "a = 1 + b = 2");

		map.put("a", new Double(100));
		map.put("b", new Double(10));
		testOpt("a+=b+=1", map, "a += b += 1");
		assertVar(100, map, "a");
		assertVar(10, map, "b");

		map.put("a", new Double(100));
		map.put("b", new Double(10));
		testOpt("a-=b-=1", map, "a -= b -= 1");
		assertVar(100, map, "a");
		assertVar(10, map, "b");

		map.put("a", new Double(2));
		map.put("b", new Double(3));
		testOpt("a*=b*=4", map, "a *= b *= 4");
		assertVar(2, map, "a");
		assertVar(3, map, "b");

		map.put("a", new Double(24));
		map.put("b", new Double(6));
		testOpt("a/=b/=2", map, "a /= b /= 2");
		assertVar(24, map, "a");
		assertVar(6, map, "b");

		map.put("a", new Double(10));
		map.put("b", new Double(10));
		testOpt("a%=b%=6", map, "a %= b %= 6");
		assertVar(10, map, "a");
		assertVar(10, map, "b");

		map.put("a", new Double(1));
		map.put("b", new Double(2));
		testOpt("a<<=b<<=1", map, "a <<= b <<= 1");
		assertVar(1, map, "a");
		assertVar(2, map, "b");

		map.put("a", new Double(1 << 8));
		map.put("b", new Double(1 << 4));
		testOpt("a>>=b>>=3", map, "a >>= b >>= 3");
		assertVar(1 << 8, map, "a");
		assertVar(1 << 4, map, "b");

		map.put("a", new Double(-1));
		map.put("b", new Double(-1));
		testOpt("a>>>=b>>>=63", map, "a >>>= b >>>= 63");
		assertVar(-1, map, "a");
		assertVar(-1, map, "b");

		map.put("a", new Double(0x0f7f));
		map.put("b", new Double(0x0ff0));
		testOpt("a&=b&=255", map, "a &= b &= 255");
		assertVar(0xf7f, map, "a");
		assertVar(0xff0, map, "b");

		map.put("a", new Double(0xf000));
		map.put("b", new Double(0x0f00));
		testOpt("a|=b|=255-15", map, "a |= b |= 240.0");
		assertVar(0xf000, map, "a");
		assertVar(0x0f00, map, "b");

		map.put("a", new Double(0xffff));
		map.put("b", new Double(0x0f0f));
		testOpt("a^=b^=255", map, "a ^= b ^= 255");
		assertVar(0xffff, map, "a");
		assertVar(0x0f0f, map, "b");
	}

	public void testComma() {
		testOpt("1,2,3", null, 3);
		testOpt("1,a,3", null, "a, 3");
		Map map = new HashMap();
		testOpt("aa=1,bb=0,cc=2,aa+bb+cc", map,
				"aa = 1, bb = 0, cc = 2, aa + bb + cc");
		// assertVar(1, map, "aa");
		// assertVar(0, map, "bb");
		// assertVar(2, map, "cc");
	}

	public void testIncBefore() {
		Map map = new HashMap();
		map.put("i", new Double(1));
		testOpt("++i", map, "++i");
		// assertVar(2, map, "i");
		testOpt("--i", map, "--i");
		// assertVar(1, map, "i");
		testOpt("2+ ++i", map, "2 + ++i");

		testOpt("++--i", map, "++ --i");
	}

	public void testIncAfter() {
		Map map = new HashMap();
		map.put("i", new Double(1));
		testOpt("i++", map, "i++");
		// assertVar(2, map, "i");
		testOpt("i--", map, "i--");
		// assertVar(1, map, "i");
		testOpt("2+ i++", map, "2 + i++");
		// assertVar(2, map, "i");

		testOpt("i++--", map, "i++ --");
		testOpt("++i++", map, "++i++");
		testOpt("++i--", map, "++i--");
	}

	public void testFunction() {
		testOpt("test()", null, "test()");
		testOpt("zzz()+foo(1)+test(2,3+4,5*6-7)", null,
				"zzz() + foo(1) + test(2, 7.0, 23.0)");

		testOpt("abs(-9)", null, "abs(-9.0)");
		testOpt("max(2,3)", null, "max(2, 3)");
		testOpt("min(2,3)", null, "min(2, 3)");
		testOpt("1+round(100)", null, "1 + round(100)");
	}

	public void testField() {
		Map map = new HashMap();
		map.put("a", new FieldTestA(1.1));
		testOpt("a.n", null, "a.n");
		testOpt("a.n", map, 1.1);
	}

	/**
	 * ñeXg.
	 * 
	 * @param str
	 *            
	 * @param map
	 *            ϐ}bv
	 * @param expected
	 *            
	 */
	private void testOpt(String str, Map map, double expected) {
		System.out.println("" + str + "");
		Expression exp = ExpRuleFactory.getDefaultRule().parse(str);
		// exp.dump(0);
		System.out.println(exp.toString());
		MapVariable var = null;
		if (map != null) {
			var = new MapVariable();
			var.setMap(map);
		}

		String old = exp.toString();

		Expression opt = exp.dup();
		opt.optimizeDouble(var);
		double ret = Double.parseDouble(opt.toString());
		System.out.println("= " + ret);
		assertEquals(expected, ret, DELTA);

		assertEquals(old, exp.toString());
	}

	/**
	 * ñeXg.
	 * 
	 * @param str
	 *            
	 * @param map
	 *            ϐ}bv
	 * @param expected
	 *            
	 */
	private void testOpt(String str, Map map, String expected) {
		System.out.println("" + str + "");
		Expression exp = ExpRuleFactory.getDefaultRule().parse(str);
		// exp.dump(0);
		System.out.println(exp.toString());
		MapVariable var = null;
		if (map != null) {
			var = new MapVariable();
			var.setMap(map);
		}

		String old = exp.toString();

		Expression opt = exp.dup();
		opt.optimizeDouble(var);
		String ret = opt.toString();
		System.out.println("= " + ret);
		assertEquals(expected, ret);

		assertEquals(old, exp.toString());
	}

	/**
	 * ُñeXg.
	 * <p>
	 * ]EvalExceptionN͂B<br>
	 * ibZ[WǂɂẴ`FbN͏ȗj
	 * </p>
	 * 
	 * @param str
	 *            
	 * @param map
	 *            ϐ}bv
	 */
	public void testOpt(String str, Map map) {
		System.out.println("" + str + "");
		Expression exp = ExpRuleFactory.getDefaultRule().parse(str);
		// exp.dump(0);
		System.out.println(exp.toString());
		MapVariable var = null;
		if (map != null) {
			var = new MapVariable();
			var.setMap(map);
		}

		try {
			exp.optimizeDouble(var);
			System.out.println("= " + exp.toString());
		} catch (EvalException e) {
			System.out.println(e.toString());
			return;
		}
		fail();
	}

	/**
	 * ϐl̊mF.
	 * 
	 * @param expected
	 *            z肳l
	 * @param map
	 *            ϐ}bv
	 * @param name
	 *            ϐ
	 */
	private void assertVar(double expected, Map map, String name) {
		try {
			Double var = (Double) map.get(name);
			assertEquals(expected, var.doubleValue(), DELTA);
		} catch (Exception e) {
			throw new RuntimeException("assertVar(" + name + ")", e);
		}
	}

	/** double̔rɎgp臒l. */
	private static final double DELTA = Math.pow(10, -15);

}
