package jp.hishidama.eval.ref;

import jp.hishidama.eval.*;
import jp.hishidama.eval.exp.*;
import jp.hishidama.eval.srch.SearchAdapter;
import jp.hishidama.eval.var.MapVariable;
import junit.framework.TestCase;

/**
 * t@N^OigetterύXj̃eXg.
 * 
 * @author <a target="hishidama"
 *         href="http://www.ne.jp/asahi/hishidama/home/soft/java/eval.html">Ђ</a>
 */
public class RefactorGetterTest extends TestCase {

	/*
	 * ϐύXeXgE\bh
	 */
	public void testVar() {
		Rule rule = ExpRuleFactory.getDefaultRule();
		Expression exp = rule.parse("a=x");

		assertEquals("a = x", exp.toString());
		exp.refactorFunc(new RefactorVarName(null, "x", "getX()"), rule);
		assertEquals("a = getX()", exp.toString());

		exp = rule.parse("a=x+y+x");
		assertEquals("a = x + y + x", exp.toString());
		exp.refactorFunc(new RefactorVarName(null, "x", "getX()"), rule);
		assertEquals("a = getX() + y + getX()", exp.toString());
		exp.refactorFunc(new RefactorVarName(null, "y", "getY()"), rule);
		assertEquals("a = getX() + getY() + getX()", exp.toString());

		exp = rule.parse("x+=x=x+x()");
		assertEquals("x += x = x + x()", exp.toString());
		exp.refactorFunc(new RefactorVarName(null, "x", "getX()"), rule);
		assertEquals("x += x = getX() + x()", exp.toString());
	}

	public void testField() {
		Rule rule = ExpRuleFactory.getDefaultRule();
		Expression exp = rule.parse("a=x+b.x");
		MapVariable var = new MapVariable();
		var.setValue("b", new FieldTestA(1));
		exp.setVariable(var);

		assertEquals("a = x + b.x", exp.toString());
		exp.refactorFunc(new RefactorVarName(FieldTestA.class, "x", "getX()"),
				rule);
		assertEquals("a = x + b.getX()", exp.toString());

		exp = rule.parse("b.x+=b.x=b.x()+b.x+a.x+x");
		exp.setVariable(var);
		assertEquals("b.x += b.x = b.x() + b.x + a.x + x", exp.toString());
		exp.refactorFunc(new RefactorVarName(FieldTestA.class, "x", "getX()"),
				rule);
		assertEquals("b.x += b.x = b.x() + b.getX() + a.x + x", exp.toString());
	}

	public void testChangeOperator() {
		Rule rule = ExpRuleFactory.getDefaultRule();
		MapVariable var = new MapVariable();
		var.setValue("a", new FieldTestA(1));

		Expression exp = rule.parse("a.x ** 2");
		exp.setVariable(var);
		assertEquals("a.x ** 2", exp.toString());
		exp.refactorFunc(new RefactorVarName(FieldTestA.class, "x",
				"getX(2**3)"), rule);
		assertEquals("a.getX(2 ** 3) ** 2", exp.toString());

		exp.search(new SearchAdapter() {
			// ݏ扉Zq́u**vu^vɕς
			public void search(AbstractExpression exp) {
				if (exp instanceof PowerExpression) {
					exp.setOperator("^");
				}
			}
		});
		assertEquals("a.getX(2 ^ 3) ^ 2", exp.toString());
	}
}
